﻿<# 
.APPNAME
    ArcheRage-chat
.VER
    1.6
.AUTHOR
    Otu
.FOR
    BAYRAKTAR
#>

Add-Type -AssemblyName System.Windows.Forms
[System.Windows.Forms.Application]::EnableVisualStyles()

function Set-VariablesFromConfig {
    param (
        [Parameter(Mandatory=$true)]
        [string]$FilePath
    )

    if (-not (Test-Path $FilePath)) {
        throw "File not found: $FilePath"
    }

    $content = Get-Content $FilePath
    foreach ($line in $content) {
        $variableName = $line.Split('=')[0].Trim()
        $variableValue = $line.Split('=')[1].Trim()
        if ($variableValue -eq 'true'){
            $variableValue = $true
        }elseif ($variableValue -eq 'false'){
            $variableValue = $false
        }
        New-Variable -Name $variableName -Value $variableValue -Force -Scope Script
    }
}

Set-VariablesFromConfig -FilePath ".\config.txt"

function Save-VariablesInConfig{
$newConfig = @"
appLang=$appLang
appFontSize=$appFontSize
appOpacity=$( $Form.Opacity )
appMostTop=$( $Form.TopMost )
appWidth=$( $Form.ClientSize.Width )
appHeight=$( $Form.ClientSize.Height )
appLeft=$( $Form.Left )
appTop=$( $Form.Top )
appAPIkey=$appAPIkey
appAPIsecret=$appAPIsecret
"@
$newConfig | Set-Content -Path ".\config.txt"
}

if (-not $appLang -or [string]::IsNullOrEmpty($appLang)) {
    $appLang = "en"
}
if (-not $appFontSize -or [string]::IsNullOrEmpty($appFontSize)) {
    $appFontSize = 11
}
if (-not $appOpacity -or [string]::IsNullOrEmpty($appOpacity)) {
    $appOpacity = 1
}
if (-not $appMostTop -or [string]::IsNullOrEmpty($appMostTop)) {
    $appMostTop = $false
}
if (-not $appWidth -or [string]::IsNullOrEmpty($appWidth)) {
    $appWidth = 600
}
if (-not $appHeight -or [string]::IsNullOrEmpty($appHeight)) {
    $appHeight = 350
}
if (-not $appLeft -or [string]::IsNullOrEmpty($appLeft)) {
    $appLeft = 100
}
if (-not $appTop -or [string]::IsNullOrEmpty($appTop)) {
    $appTop = 200
}
if (-not $appAPIkey) {
    $appAPIkey = ""
}
if (-not $appAPIsecret) {
    $appAPIsecret = ""
}

$styleFontFamilyChat = 'Consolas'
$styleFontSizeChat = 12
$screenWidth = [System.Windows.Forms.Screen]::PrimaryScreen.WorkingArea.Width
$screenHeight = [System.Windows.Forms.Screen]::PrimaryScreen.WorkingArea.Height

$Form                            = New-Object system.Windows.Forms.Form
$Form.ClientSize                 = New-Object System.Drawing.Size($appWidth,$appHeight)
$Form.MinimumSize                = New-Object System.Drawing.Size(300, 200)
$Form.StartPosition              = "Manual"
$Form.Location                   = New-Object System.Drawing.Point($appLeft, $appTop)
$Form.text                       = "ArcheRage chat translator (by Otu)"
$Form.TopMost                    = $false
$Form.FormBorderStyle            = 'Sizable'
$Form.MaximizeBox                = $false
$Form.BackColor                  = [System.Drawing.ColorTranslator]::FromHtml("#151515")
$Form.Icon                       = [System.Drawing.Icon]::ExtractAssociatedIcon('.\archerage.ico')
$Form.Opacity                    = $appOpacity
$Form.Add_Shown({ $Form.Activate(); $ConsoleChatBox.focus() })
$Form.Add_Closing({ stopTranslate })

$ToolTip                         = New-Object System.Windows.Forms.ToolTip

$checkBox                        = New-Object System.Windows.Forms.CheckBox
$checkBox.Location               = New-Object System.Drawing.Point(($Form.ClientSize.Width - 40), 5)
$checkBox.Size                   = New-Object System.Drawing.Size(37, 20)
$checkBox.Text                   = [char]::ConvertFromUtf32(0x1F432)
$checkBox.Font                   = New-Object System.Drawing.Font('Segoe UI Emoji', 10)
$checkBox.ForeColor              = [System.Drawing.ColorTranslator]::FromHtml("#FF0000")
$ToolTip.SetToolTip($checkBox, "Use youdao.com API")

$mostTop                         = New-Object System.Windows.Forms.CheckBox
$mostTop.Location                = New-Object System.Drawing.Point(($Form.ClientSize.Width - 80), 5)
$mostTop.Size                    = New-Object System.Drawing.Size(37, 20)
$mostTop.Text                    = [char]::ConvertFromUtf32(0x1F4CC)
$mostTop.Font                    = New-Object System.Drawing.Font('Segoe UI Emoji', 10)
$mostTop.ForeColor               = [System.Drawing.ColorTranslator]::FromHtml("#FFFFFF")
$ToolTip.SetToolTip($mostTop, "Always on top")

if ($appMostTop -eq $true) {
		$mostTop.Checked = $true
        $Form.TopMost = $true
}else {
	$mostTop.Checked = $false
    $Form.TopMost = $false
}

$mostTop.Add_Click({
    if ($mostTop.Checked) {
        $Form.TopMost = $true
    } else {
        $Form.TopMost = $false
    }
    Save-VariablesInConfig
})

$opacity                        = New-Object System.Windows.Forms.ComboBox
$opacity.Location               = New-Object System.Drawing.Point((($Form.ClientSize.Width / 2) + 10), 0)
$opacity.Size                   = New-Object System.Drawing.Size((($Form.ClientSize.Width / 2) - 100), 20)
$opacity.Font                   = New-Object System.Drawing.Font($styleFontFamilyChat,$styleFontSizeChat)
$opacity.DropDownStyle          = 'DropDownList'
$opacity.Items.AddRange(@('Opacity', '90%', '75%', '60%', '50%', '35%', '25%'))
$opacity.SelectedIndex = 0

$opacity.Add_SelectedIndexChanged({
    $selectedOpacity = $opacity.SelectedItem.ToString()
    if ($selectedOpacity -eq '90%') {
        $Form.Opacity = 0.9
    }elseif ($selectedOpacity -eq '75%') {
        $Form.Opacity = 0.75
    }elseif ($selectedOpacity -eq '60%') {
        $Form.Opacity = 0.6
    }elseif ($selectedOpacity -eq '50%') {
        $Form.Opacity = 0.5
    }elseif ($selectedOpacity -eq '35%') {
        $Form.Opacity = 0.35
    }elseif ($selectedOpacity -eq '25%') {
        $Form.Opacity = 0.25
    }else {
        $Form.Opacity = 1
    }
    Save-VariablesInConfig
})

$comboBox                        = New-Object System.Windows.Forms.ComboBox
$comboBox.Location               = New-Object System.Drawing.Point(0, 0)
$comboBox.Size                   = New-Object System.Drawing.Size(($Form.ClientSize.Width / 2), 20)
$comboBox.Font                   = New-Object System.Drawing.Font($styleFontFamilyChat,$styleFontSizeChat)
$comboBox.DropDownStyle          = 'DropDownList'

$comboBox.Items.AddRange(@('Choose your language', 'English EN', 'Polski PL', '中文 ZH-CN', 'Українська UK', 'Français FR', 'Español ES', 'Português PT-BR', 'Türkçe TR', 'Русский RU'))

$comboBox.SelectedIndex = 0

$comboBox.Add_SelectedIndexChanged({
    $selectedOption = $comboBox.SelectedItem.ToString()
    if ($selectedOption -eq 'English EN') {
        startTranslate -lang 'en'
        $appLang = 'en'
    }elseif ($selectedOption -eq 'Polski PL') {
        startTranslate -lang 'pl'
        $appLang = 'pl'
    }elseif ($selectedOption -eq '中文 ZH-CN') {
        startTranslate -lang 'zh-cn'
        $appLang = 'zh-cn'
    }elseif ($selectedOption -eq 'Українська UK') {
        startTranslate -lang 'uk'
        $appLang = 'uk'
    }elseif ($selectedOption -eq 'Français FR') {
        startTranslate -lang 'fr'
        $appLang = 'fr'
    }elseif ($selectedOption -eq 'Español ES') {
        startTranslate -lang 'es'
        $appLang = 'es'
    }elseif ($selectedOption -eq 'Português PT-BR') {
        startTranslate -lang 'pt'
        $appLang = 'pt'
    }elseif ($selectedOption -eq 'Türkçe TR') {
        startTranslate -lang 'tr'
        $appLang = 'tr'
    }elseif ($selectedOption -eq 'Русский RU') {
        startTranslate -lang 'ru'
        $appLang = 'ru'
    }
    Save-VariablesInConfig
})

$ConsoleChatBox = New-Object System.Windows.Forms.RichTextBox
$ConsoleChatBox.Multiline = $true
$ConsoleChatBox.ScrollBars = "Vertical"
$ConsoleChatBox.BackColor = [System.Drawing.ColorTranslator]::FromHtml("#131212")
$ConsoleChatBox.Width = ($Form.ClientSize.Width - 2)
$ConsoleChatBox.Height = ($Form.ClientSize.Height - 28)
$ConsoleChatBox.Location = New-Object System.Drawing.Point(1,27)
$ConsoleChatBox.Font = New-Object System.Drawing.Font($styleFontFamilyChat, $appFontSize)
$ConsoleChatBox.ForeColor = [System.Drawing.ColorTranslator]::FromHtml("#FFFFFF")

$ConsoleChatBox.AppendText("[System: Otu]: Hi! o/`n")
$ConsoleChatBox.SelectionColor = "Red"
$ConsoleChatBox.AppendText("Choose your language`n")

function UpdateSizeAndLocation() {
    $comboBox.Size = New-Object System.Drawing.Size(($Form.ClientSize.Width / 2), 20)
    $ConsoleChatBox.Size = New-Object System.Drawing.Size(($Form.ClientSize.Width - 2), ($Form.ClientSize.Height - 28))
    $checkBox.Location = New-Object System.Drawing.Point(($Form.ClientSize.Width - 40), 5)
    $mostTop.Location = New-Object System.Drawing.Point(($Form.ClientSize.Width - 80), 5)
    $opacity.Location = New-Object System.Drawing.Point((($Form.ClientSize.Width / 2) + 10), 0)
    $opacity.Size = New-Object System.Drawing.Size((($Form.ClientSize.Width / 2) - 100), 20)
    $ConsoleChatBox.ScrollToCaret()
    Save-VariablesInConfig
}

$Form.add_SizeChanged({
    UpdateSizeAndLocation
})

if (![string]::IsNullOrEmpty($appAPIkey) -and ![string]::IsNullOrEmpty($appAPIsecret)) {
		$checkBox.Checked = $true
		$Global:googleAPI=$false
}else {
	$checkBox.Checked = $false
	$Global:googleAPI=$true
}

$checkBox.Add_Click({
    if ($checkBox.Checked) {
        stopTranslate
        $Global:googleAPI=$false
		
    } else {
        stopTranslate
        $Global:googleAPI=$true
    }
})

$Global:fileSizeMEM=0
$scriptBlock = {
	$Global:fileNumLinesMem = 0
	$Global:failTranslate = 0
	$pathFile = [Environment]::GetFolderPath("MyDocuments")+'\ArcheRage\Chat.log'
    if (Test-Path $pathFile){
		$fileNumLines = (Get-Content $pathFile | Measure-Object -Line).Lines
		if ($Global:googleAPI -eq $true) {
			$maxLineFromFile = 30
		}else {
			$maxLineFromFile = 5
		}
		if ($fileNumLines -gt $maxLineFromFile) {
			$Global:fileNumLinesMem = $fileNumLines - $maxLineFromFile
		} else {
			$Global:fileNumLinesMem = 1
		}
	}
    function DecodeUnicodeAndTrimQuotes {
        Param ([string]$s)
        return [regex]::Unescape($s).Trim('"')
    }
    function translateThisGoogle(){
        param(
            [Parameter()]
            [string]$translate,

            [Parameter()]
            [string]$text
        )
		
		if ($text -like "* ") {
			$text = $text.TrimEnd()
		}
		
        if ($Global:googleAPI -eq $true) {
            $Uri = "https://translate.googleapis.com/translate_a/single?client=gtx&sl=auto&tl=$($translate)&dt=t&q=$text"
            $Response = Invoke-RestMethod -Uri $Uri -Method Get
		    return $Translation = $Response[0].SyncRoot | foreach { $_[0] }
        }else {
            if (![string]::IsNullOrEmpty($appAPIkey) -and ![string]::IsNullOrEmpty($appAPIsecret)) {
				$salt = [string][int]([DateTime]::Now - [DateTime]::ParseExact("1970-01-01", "yyyy-MM-dd", $null)).TotalSeconds
				$sign = [System.Web.Security.FormsAuthentication]::HashPasswordForStoringInConfigFile("$appAPIkey$text$salt$appAPIsecret", "MD5").ToLower()
				
				$Uri = "https://openapi.youdao.com/api?from=auto&to=$($translate)&appKey=$appAPIkey&salt=$salt&sign=$sign&q=$text"
				$Response = Invoke-RestMethod -Uri $Uri -Method Get
				
				if ($Response.translation -eq $null -or $Response.translation.Count -eq 0) {
					$Translation = "[API ERROR CODE]: "+$Response.errorCode
				}else {
					$Translation = $Response.translation[0]
				}
				return $Translation
            }
        }
    }

    function loadChat(){
        param(
             [Parameter()]
             [string]$lang
        )
        $pathFile = [Environment]::GetFolderPath("MyDocuments")+'\ArcheRage\Chat.log'
        if (Test-Path $pathFile){
			$fileNumLines = (Get-Content $pathFile | Measure-Object -Line).Lines
            if ($fileNumLines -gt $Global:fileNumLinesMem){
                $final = New-Object System.Collections.Generic.List[System.String]
				$text = (Get-Content -Path $pathFile -Encoding UTF8)[$Global:fileNumLinesMem]
				$Global:fileNumLinesMem += 1
				$text = $text -replace '\+', 'x452.'
				$text = $text -replace ']:]:', ']:'
                $chat = $text -replace '(?<=<).*?(?=> )', '' -replace '<>', ''
                ForEach ($line in $($chat -split "`r`n")){
                    $Line = $Line -split "]:"
                    $player=$Line[0];
                    $textT = $Line[1]
                    $textT = $textT -replace '\|n(.+?)\|r', '{NAME}'
                    $textT = $textT -replace '\|A(.+?)\;', '{INSTANCE}'
                    $textT = $textT -replace '\|i(.+?)\;', '{LINK}'
                    $translate = translateThisGoogle -translate $lang -text $textT
                    $text = DecodeUnicodeAndTrimQuotes -s $translate
                    $text = $text -replace 'x452.', '+'
                    $newLine=$player+']: '+$text
                    $final.Add($newLine)
                }
                
                foreach ($line in $final) {
                    $echo = ''
                    if ($line.Contains("[API ERROR CODE]")) {
                        $color = "#FF0000"
						if ($Global:failTranslate -lt 3) {
							$Global:fileNumLinesMem -= 1
							$line = $line+" (I'm trying again)"
							$Global:failTranslate += 1
						}else {
							$line = $line+" (3x failures, I'm skipping it)"
							$Global:failTranslate = 1
						}
                    }else{
						$Global:failTranslate = 1
						if ($line.Contains("[Nation:") -or $line.Contains("[势力:") -or $line.Contains("[Союз:") -or $line.Contains("[Nation :")) {
							$color = "#8FB232"
						}elseif ($line.Contains("[Trade:") -or $line.Contains("[交易:") -or $line.Contains("[Торговля:") -or $line.Contains("[Échanger:")) {
							$color = "#36F1CC"
						}elseif ($line.Contains("[Raid:") -or $line.Contains("[团队") -or $line.Contains("[Рейд:") -or $line.Contains("[Raid :")) {
							$color = "#EB8B2D"
						}elseif ($line.Contains("[Guild:") -or $line.Contains("[远征队") -or $line.Contains("[Гильдия:") -or $line.Contains("[Guilde :")) {
							$color = "#659FFF"
						}elseif ($line.Contains("[To ") -or $line.Contains("[你对") -or $line.Contains("[Вы говорите герою ") -or $line.Contains("[À ")) {
							$color = "#EA6DDA"
						}elseif ($line.Contains("to you: ]") -or $line.Contains("对你：]") -or $line.Contains("говорит вам: ]") -or $line.Contains("à vous :")) {
							$color = "#EA6DDA"
						}elseif ($line.Contains("[Party:") -or $line.Contains("[队伍") -or $line.Contains("[Отряд:") -or $line.Contains("[Groupe :")) {
							$color = "#6BEF80"
						}elseif ($line.Contains("[Team:") -or $line.Contains("[战队") -or $line.Contains("[Группа:") -or $line.Contains("[Équipe :")) {
							$color = "#DEA8F4"
						}elseif ($line.Contains("[Family:") -or $line.Contains("[家族") -or $line.Contains("[Семья:") -or $line.Contains("[Famille :")) {
							$color = "#1FD857"
						}elseif ($line.Contains("[Shout:") -or $line.Contains("[场景") -or $line.Contains("[Крик:") -or $line.Contains("[Crier:")) {
							$color = "#F86C96"
						}elseif ($line.Contains("[Trial:") -or $line.Contains("[审判") -or $line.Contains("[Суд:") -or $line.Contains("[Procès :")) {
							$color = "#EFAA25"
						}elseif ($line.Contains("[Commander:") -or $line.Contains("[指挥") -or $line.Contains("[Commandant ") -or $line.Contains("[Глава отряда")) {
							$color = "#FF7D1D"
						}elseif ($line.Contains("[Global:") -or $line.Contains("[跨服") -or $line.Contains("[Global :") -or $line.Contains("[Общий чат:")) {
							$color = "#35F0CB"
						}else {
							$color = "#FFFFFF"
						}
					}
                    if (!($line.StartsWith("BackupNameAttachment"))) {
                        $echo += "`n"+$line
                        $ConsoleChatBox.SelectionColor = [System.Drawing.ColorTranslator]::FromHtml($color)
                        $ConsoleChatBox.AppendText( $echo )
                    }
                }
                Remove-Variable -Name final
				
				if ($Global:googleAPI -eq $true) {
					$maxLines = $maxLineFromFile
				}else {
					$maxLines = 25
				}
				if ($ConsoleChatBox.Lines.Count -gt $maxLines) {
					$lineIndex = 0
					$lineLength = $ConsoleChatBox.GetFirstCharIndexFromLine($lineIndex + 1) - $ConsoleChatBox.GetFirstCharIndexFromLine($lineIndex)
					$ConsoleChatBox.Select($ConsoleChatBox.GetFirstCharIndexFromLine($lineIndex), $lineLength)
					$ConsoleChatBox.SelectedText = ""
				}
				
                $ConsoleChatBox.SelectionStart = $ConsoleChatBox.Text.Length
                $ConsoleChatBox.ScrollToCaret()
            }
        }
    }
	For ($i = 0; $i -lt 25000; $i++) {
		loadChat -lang $tToLang
		if ($fileNumLines -ne $Global:fileNumLinesMem){
			if ($Global:googleAPI -eq $true) {
				Start-Sleep -Milliseconds 250
			}else {
				start-sleep -Seconds 1
			}
		}else {
			start-sleep -Seconds 2
		}
	}
}

$Global:ps2 = [powershell]::create()
function startTranslate(){
    param(
        [Parameter()]
        [string]$lang
    )
    $Global:tToLang=$lang
    stopTranslate
    $Global:ps2 = [powershell]::create()
    $Global:ps2.AddScript($scriptBlock)
    $Global:ps2.Runspace.SessionStateProxy.SetVariable("ConsoleChatBox", $ConsoleChatBox)
    $Global:ps2.Runspace.SessionStateProxy.SetVariable("tToLang", $Global:tToLang)
    $Global:ps2.Runspace.SessionStateProxy.SetVariable("googleAPI", $Global:googleAPI)
    $Global:ps2.Runspace.SessionStateProxy.SetVariable("appAPIkey", $appAPIkey)
	$Global:ps2.Runspace.SessionStateProxy.SetVariable("appAPIsecret", $appAPIsecret)
    $Global:ps2.BeginInvoke()
}
function stopTranslate(){
    $Global:ps2.Stop()
    $Global:ps2.Dispose()
	$ConsoleChatBox.Clear()
}

    if (![string]::IsNullOrEmpty($appLang)) {
        startTranslate -lang $appLang
    }

$Form.controls.AddRange(@($comboBox,$checkBox,$mostTop,$opacity,$ConsoleChatBox))
[void]$Form.ShowDialog()